import 'dart:async';

import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';
import 'package:kameleoon_client_flutter/kameleoon_client_factory.dart';
import 'package:kameleoon_client_flutter/kameleoon_data.dart';

import 'constants.dart';

String makeReason(String text) => "\n\n\n" + ERROR_REASON + "\n" + text + "\n \n \n";

void main() {
  const visitorCode = "222D7B50-A0C7-435B-B628-E86670DDBDB9";

  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  final client = KameleoonClientFactory.create(siteCode)!;

  const countOfRuns = 100;
  const millisecondsBetweenRuns = 10;

  test(Function(int, Completer) testFunc) async {
    final completer = new Completer();
    client.runWhenReady(() async {
      for(var i = 0; i < countOfRuns + 1; i++) {
        await Future.delayed(const Duration(milliseconds: millisecondsBetweenRuns));
        testFunc.call(i, completer);
      }
    }, () {
      completer.completeError(TIMEOUT_FAIL_ERROR);
    }, TIMEOUT_MILLISECONDS);

    await expectLater(completer.future, completion(equals(COMPLETE_VALID)), reason: makeReason("runWhenReady Timeout"));
  }

  testWidgets("multiRequest", (WidgetTester tester) async {
    test((currentRun, completer) async {
      const experimentId_1 = 136200, experimentId_2 = 136204, experimentId_3 = 136226;
      try {
        await client.triggerExperiment(visitorCode, experimentId_1);
        await client.triggerExperiment(visitorCode, experimentId_2);
        await client.triggerExperiment(visitorCode, experimentId_3);
      } catch (e) {}
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

   test((currentRun, completer) async {
      const mockedId = 139626;
      try {
        await client.activateFeature(visitorCode, mockedId);
      } catch (e) {}
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      const featureKey = "test-feature-variables";
      try {
        await client.activateFeature(visitorCode, featureKey);
      } catch (e) {}
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      const featureKey = "test-feature-variables";
      try {
        await client.activateFeature(visitorCode, featureKey);
      } catch (e) {}
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      await client.addData(visitorCode, CustomData(30 + currentRun, "12345678"));
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      await client.flush();
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      const mockedVariationId = "toto";
      const mockedId = 139626;
      const mockedExperimentId = 136200;
      final customData = CustomData(2, "true");
      client.runWhenReady(() async {
        try {
          await client.obtainFeatureVariable(mockedId, mockedVariationId);
          await client.activateFeature(visitorCode, mockedId);
          await client.triggerExperiment(visitorCode, mockedExperimentId);
          await client.addData(visitorCode, customData);
          await client.flush();
          client.runWhenReady(() async {
            try {
              await client.obtainFeatureVariable(mockedId, mockedVariationId);
              await client.activateFeature(visitorCode, mockedId);
              await client.triggerExperiment(visitorCode, mockedExperimentId);
              await client.addData(visitorCode, customData);
              await client.flush();
            } catch (e) {}
          }, () {
            completer.completeError(TIMEOUT_FAIL_ERROR);
          });
        } catch (e) {}
      }, () {
        completer.completeError(TIMEOUT_FAIL_ERROR);
      });
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      const goalId = 238420;
      await client.trackConversion(visitorCode, goalId, 10);
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });

    test((currentRun, completer) async {
      const mockedVariationId = "toto";
      const featureKey = "testff";
      const mockedId = 139626;

      await client.obtainFeatureVariable(mockedId, mockedVariationId);
      await client.obtainFeatureVariable(featureKey, mockedVariationId);
      if (currentRun == countOfRuns) {
        completer.complete(COMPLETE_VALID);
      }
    });
  });
}